/*
UV Pipe by Robert Fletcher
Mod by Norman Schaar -- norman.schaar@gmail.com
Mod by Kostadin Kotev -- http://miauumaxscript.blogspot.com/

CHANGE LOG:
------------------------------------------------------------------------------
Mod by Kostadin Kotev -- http://miauumaxscript.blogspot.com/
6th May 2013
*Added support for selection of objects
*Added support for grouped objects
----------------------------------------------------------------
Mod by Norman Schaar -- norman.schaar@gmail.com
22nd September 2012 - by Norman
*Rewrote script to support modifiers and avoid colapssing of the stack
*Added function that attempts to automatically find uvlayout.exe
*User will now be given the choice to close UVLayout if the object was deleted while editing.
*Moved creation of temp and config files to User directories to avoid write-errors due to lack of admin rights in restricted environments, typically at work.
*Rewrote the functions so that no "temp" mesh is left behind in the max scene while editing.
*Objects with spaces in their names can now be send to UVLayout
*Removed tiling button
*Other bug fixes
----------------------------------------------------------------
19th April 2010
*Material Support Added
*New UVs not showing on second Export fixed.
-----------------------------------------------------------------
17th April 2010
*Released
-----------------------------------------------------------------
11th May 2010
*UVChannel Choicer 
*Error Checker 
*/

try(destroyDialog r_uvPipe)catch()
rollout r_uvPipe "UVPipe" width:106 height:120
(
	local uvLayoutBin
	local uvLayoutLaunchOptions = " -plugin"
	local uvLayoutTemp = sysInfo.tempdir + "\\"
	local uvLayoutNodes
	local uvLayoutName
	
	local meshModes = #("Poly", "SUBD")
	local uvModes = #("New", "Edit")
	
	dropdownList ddl1 "UV Ch." pos:[6,3] width:40 height:40 items:#("1", "2", "3")
	radiobuttons rb_meshMode "Mode:" pos:[55,1] width:50 height:46 labels:meshModes default:1 columns:1	
	radiobuttons rb_uvMode "UV Mode:" pos:[6,49] width:45 height:46 labels:uvModes default:1 columns:2
	button btn_sendSelection "Send Selection" height:30 align:#center offset:[0,-15]
	timer tmr_checkForUvs "wait" pos:[-145,-113] width:24 height:24 interval:500 active:false
	
	fn GET_UVLAYOUTPATH =
	(
		local UVLAYOUT_PATH_KEY
		local UVLAYOUT_PATH_VALNAME
		local UVLAYOUT_PATH_VALUE
		local UVLAYOUT_PATH_TYPE
		local THE_KEY = registry.openKey HKEY_CLASSES_ROOT "uvl.File\\shell\\open\\command\\" accessRights:#readOnly key:&UVLAYOUT_PATH_KEY
		
		if THE_KEY == true then
		(
			registry.getValueName UVLAYOUT_PATH_KEY 1 name:&UVLAYOUT_PATH_VALNAME
			registry.queryValue UVLAYOUT_PATH_KEY UVLAYOUT_PATH_VALNAME type:&UVLAYOUT_PATH_TYPE value:&UVLAYOUT_PATH_VALUE
			registry.closeKey UVLAYOUT_PATH_KEY
			local thePath = substituteString UVLAYOUT_PATH_VALUE "\"" ""
			thePath = substituteString thePath " %1" ""
			thePath = (pathconfig.normalizepath thePath)
			if doesFileExist thePath then
			(
				return thePath
			)
			else
			(
				return false
			)
		)
		else
		(
			return false
		)
	)
	
	fn ExportObj nNodes fileName = 
	(			
		local matLib = #()
		local materialList =#()
		local numMaterials = 1
		local filenameStub = filterString filename "."
		local mtlFileName = filenameStub[1] + ".mtl"		
		local currentMtl = ""
			
		fn safeMatName matName =
		(
			filteredMatName = filterString matName " -"
			matName = ""
			for element in filteredMatName do
			(
				matName = MatName + element
			)
			matName
		)			
		
		local file = createfile fileName
		if file != undefined then
		(
			local get_vert = polyop.getVert
			local get_tVert = polyOp.getMapVert
			local get_face_verts = polyOp.getFaceVerts
			local get_map_face = polyOp.getMapFace 
			local get_face_ID = polyOp.getFaceMatID
				
			format "# 3DS Max to UVLayout OBJ Exporter\n" to:file 
			format "\n" to:file		
			format "mtlLib %.mtl\n" (getFilenameFile mtlFileName) to:file
			--	count verts
			local vCnt = 0
			local tvCnt = 0
			for nNode in nNodes do
			(
				local mapChan = (ddl1.selected as integer)
				local numFaces = polyop.getNumFaces nNode
				local numVerts = polyop.getNumVerts nNode
				local numTVerts = polyop.getNumMapVerts nNode mapChan		
					
				if file != undefined then
				(
					format "\n" to:file
					format "#\n" to:file
					format "# object % \n" nNode.name to:file
					format "#\n" to:file
					format "\n" to:file					
					
					for i = 1 to numVerts do
					(
						vert = get_vert nNode i
						format "v % % %\n" vert[1] vert[2] vert[3] to:file
					)
					format "# % vertices\n" numVerts to:file
					format "\n" to:file
					
					for i = 1 to numTVerts do
					(
						tVert = get_tVert nNode mapChan i
						format "vt % % %\n" tVert[1] tVert[2] tVert[3] to:file
					)
					format "# % texture coords\n" numTVerts to:file					
					format "\n" to:file
					
					format "g %\n" nNode.name to:file
					local clr =( nNode.wirecolor )
					local clrS = (((clr.r as integer) as string) + ((clr.g as integer) as string) + ((clr.b as integer) as string)) as integer
					format "usemtl wire_%\n" clrS to:file
-- 					format "s %\n" 1 to:file	--	smoothing group
					for i = 1 to numFaces do
					(
						faceVerts = get_face_verts nNode  i
						faceMapVerts = get_map_face nNode mapChan i
						faceVertCount = faceVerts.count
						faceMatID = get_face_ID nNode i
						
						format "f" to:file
						
						for j = 1 to faceVertCount do
						(
							-- f v/vt/vn 
							format " %/%" (faceVerts[j] + vCnt) (faceMapVerts[j] + tvCnt ) to:file
-- 							format " %/%" faceVerts[j] faceMapVerts[j] to:file	--	for single object obly
						)
						format "\n" to:file
					)
					--	increase the verts and texture verts counter
					vCnt += numVerts
					tvCnt += numTVerts
				)
			)
			close file
			free file
		)
		else
			messagebox "Can't create obj file" title:"Exporting objects failed"
	)
	
	function ImportObjUvs fileName = 
	(
		with redraw off
		(
			--	resave the .out file as .obj
			objF = (getFilenamePath fileName) + (getFilenameFile fileName) +".obj"
			--	rename the new file as obj
			renameFile fileName objF
			--	get the current objects
			oldObj = objects as array
			--	import the unwraped objects
			importFile objF #noPrompt using:Wavefront_Object
			--	get imported objects
			newObj = for o in objects where ((findItem oldObj o) == 0) collect o
			--
			for o in newObj do
			(
				parentNodeHandle = (execute (filterString o.name "__")[2])
				--	get the parent node
				parentNode = undefined
				for obj in objects where (GetHandleByAnim obj) == (parentNodeHandle) do
					parentNode = obj
				--	
				if parentNode != undefined do
				(
					convertToPoly o
					channelInfo.copyChannel o 3 1	--	UVLayout returns the UV's in channel 1 --(ddl1.selected as integer)
					channelInfo.pasteChannel parentNode 3 (ddl1.selected as integer)
					parentNode.modifiers[1].name = "UVLayout UV's"
					delete o
				)
			)
		)
	) 
	
	fn UvLayoutSend nNodes = 
	(
		--This the Obj
		local fileName = (uvLayoutTemp + uvLayoutName + ".obj")
		--Map Channel
		local mapChannel = (ddl1.selected as integer)
		
		--Clean up first
		deleteFile fileName
		deleteFile (uvLayoutTemp + uvLayoutName + ".out")
		deleteFile (uvLayoutTemp + uvLayoutName + ".keep")
		
		local nodesToExport = #()
		for nNode in nNodes do
		(
			local meshSnapshot = copy nNode
			convertToPoly meshSnapshot
			meshSnapshot.name = (timestamp() as string) + "__" + (GetHandleByAnim nNode) as string
			polyop.setMapSupport meshSnapshot mapChannel true
			if mapChannel != 1 do
			(
				channelInfo.CopyChannel meshSnapshot 3 mapChannel
				channelInfo.PasteChannel meshSnapshot 3 1
			)
			append nodesToExport meshSnapshot
		)
		
		--Export the Obj
		ExportObj nodesToExport fileName
		--	Delete Temp meshes
		for n in nodesToExport do delete n
		--Launch UvLayout
		shelllaunch uvLayoutBin (" -plugin," + (meshModes)[rb_meshMode.state] + "," + (uvModes)[rb_uvMode.state] + " " + "\"" + filename + "\"")
	)

	fn UvLayoutKill = 
	(
		local cysliceP = (dotnetclass "System.Diagnostics.Process").GetProcessesByName "cysliceP"
		local cyeatP = (dotnetclass "System.Diagnostics.Process").GetProcessesByName "cyeatP"
		if cysliceP.count != 0 do (cysliceP[1].kill())
		if cyeatP.count != 0 do (cyeatP[1].kill())
	)

	fn UvLayoutRecieve nNode = 
	(
		UvLayoutKill()
		ImportObjUvs (uvLayoutTemp + uvLayoutName + ".out")
	)

	fn UvPipeInit = 
	(
		local theINIFile = "$userscripts\\UVPipe\\uvpipe_settings.ini"
		if doesFileExist theINIFile do
		(
			uvLayoutBin = getINISetting theINIFile "Path" "UVLayout"
		)
		
		if uvLayoutBin == "" OR uvLayoutBin == undefined do
		(
			uvLayoutBin = GET_UVLAYOUTPATH()
		)
		
		if uvLayoutBin == false then
		(
			local pickResult = getOpenFileName caption:"Select UVLayoutBinary" filename:"uvlayout.exe" types:"Executable(*.exe)|*.exe"
			makeDir "$userscripts\\UVPipe\\"
			if pickResult != undefined then
			(
				uvlayoutBin = pickResult
			)
			else
			(
				messagebox "UVPipe needs uvlayout.exe to work. UVPipe will now close."
				destroyDialog r_uvPipe
			)
		)
		if uvLayoutBin != false do
		(
			setINISetting theINIFile "Path" "UVLayout" uvLayoutBin
		)
	)
		
	on r_uvPipe open do	(UvPipeInit())
	
	on btn_sendSelection pressed do
	(
		if selection.count != 0 then
		(
			uvLayoutNodes = for o in selection where (classof o.baseObject == Editable_Poly) collect o
			--
			if uvLayoutNodes.count != 0 then
			(
				uvLayoutName = selection[1].name
				UvLayoutSend uvLayoutNodes
				tmr_checkForUvs.active = true
			)
			else
			(
				messagebox "Only Editable Poly objects are supported!" title:"Invalid Selection"
			)
		)
		else
			messagebox "Select some Editable Poly objects" title:"Invalid Selection"
	)

	on tmr_checkForUvs tick do
	(
		if (isDeleted uvLayoutNodes != true) then
		(
			if doesFileExist(uvLayoutTemp + uvLayoutName +".out" ) do
			(
				tmr_checkForUvs.active = false
				UvLayoutRecieve uvLayoutNodes
			)
		)
		else
		(
			local theQuestion = querybox "You have deleted the object you were editing in UVLayout. Do you want to close UVLayout?"
			if theQuestion == true do
			(
				tmr_checkForUvs.active = false
				UvLayoutKill()
			)
		)
	)
)
createDialog r_uvPipe
